"""Script for running tsunami inundation scenario for Dampier, WA, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.output_run_time_dir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a simulated tsunami generated with URS code.

Ole Nielsen and Duncan Gray, GA - 2005 and Jane Sexton, Nick Bartzis, GA - 2006
"""

#------------------------------------------------------------------------------
# Import necessary modules
#------------------------------------------------------------------------------

# Standard modules
from os import sep
from os.path import dirname, basename
from os import mkdir, access, F_OK
from shutil import copy
import time
import sys

# Related major packages
from anuga.shallow_water import Domain
from anuga.shallow_water import Dirichlet_boundary
from anuga.shallow_water import File_boundary
from anuga.shallow_water import Reflective_boundary
from Numeric import allclose

from anuga.pmesh.mesh_interface import create_mesh_from_regions
from anuga.abstract_2d_finite_volumes.util import start_screen_catcher, copy_code_files
from anuga_parallel.parallel_api import distribute, numprocs, myid, barrier

# Application specific imports
import project                 # Definition of file names and polygons

#------------------------------------------------------------------------------
# Copy scripts to time stamped output directory and capture screen
# output to file
#------------------------------------------------------------------------------

start_screen_catcher(project.output_run_time_dir, myid, numprocs)

# filenames
#boundaries_name = project.boundaries_name
meshes_dir_name = project.meshes_dir_name+'.msh'
#boundaries_dir_name = project.boundaries_dir_name

tide = project.tide

# creates copy of code in output dir
if myid == 0:
    copy_code_files(project.output_run_time_dir,__file__, 
                 dirname(project.__file__)+sep+ project.__name__+'.py' )
barrier()

print 'USER: ', project.user
print 'min triangles', project.trigs_min,
print 'Note: This is generally about 20% less than the final amount'

#--------------------------------------------------------------------------
# Create the triangular mesh based on overall clipping polygon with a
# tagged
# boundary and interior regions defined in project.py along with
# resolutions (maximal area of per triangle) for each polygon
#--------------------------------------------------------------------------
'''
poly = [[0,0],[0,100],[100,100],[100,0]]

create_mesh_from_regions(poly,
                             boundary_tags={'back': [0], 'side': [1,3],
                                            'ocean': [2]},
                             maximum_triangle_area=1,
                             interior_regions=None,
                             filename=meshes_dir_name,
                             use_cache=True,
                             verbose=True)

sys.exit()
'''
if myid == 0:
    
    print 'start create mesh from regions'
    create_mesh_from_regions(project.poly_all,
                             boundary_tags={'back': [2,3], 'side': [0, 1, 4],
                                            'ocean': [5]},
                             maximum_triangle_area=project.res_poly_all,
                             interior_regions=project.interior_regions,
                             filename=meshes_dir_name,
                             use_cache=True,
                             verbose=True)

# to sync all processors are ready 
barrier()

#-------------------------------------------------------------------------
# Setup computational domain
#-------------------------------------------------------------------------
print 'Setup computational domain'
domain = Domain(meshes_dir_name, use_cache=True, verbose=True)
print domain.statistics()
boundaries_dir_name=project.boundaries_dir_name

print 'starting to create boundary conditions'

from anuga.shallow_water.data_manager import urs2sww

# put above distribute
print 'boundary file is: ',project.boundaries_dir_name
from caching import cache
if myid == 0:
    cache(urs2sww,
          (project.boundaries_in_dir_name,
           project.boundaries_dir_name), 
          {'verbose': True,
           'minlat': project.south_boundary,
           'maxlat': project.north_boundary,
           'minlon': project.west_boundary,
           'maxlon': project.east_boundary,
           'mint': 0, 'maxt': 35100,
           'origin': domain.geo_reference.get_origin(),
           'mean_stage': project.tide,
#           'zscale': 1,                 #Enhance tsunami
           'fail_on_NaN': False},
           verbose = True,
           )
barrier()


#-------------------------------------------------------------------------
# Setup initial conditions
#-------------------------------------------------------------------------
if myid == 0:

    print 'Setup initial conditions'

    from polygon import *
    #following sets the stage/water to be offcoast only
    IC = Polygon_function( [(project.poly_bathy, 0.)], default = tide)
    domain.set_quantity('stage', IC)
    domain.set_quantity('friction', 0.01) 
    print 'Start Set quantity'

    domain.set_quantity('elevation', 
#                    filename = project.combined_dir_name + '.pts',
# MUST USE TXT FILES FOR CACHING TO WORK!
                    filename = project.combined_dir_name + '.txt',
                    use_cache = True,
                    verbose = True,
                    alpha = 0.1)
    print 'Finished Set quantity'
barrier()


#------------------------------------------------------
# Distribute domain to implement parallelism !!!
#------------------------------------------------------

if numprocs > 1:
    domain=distribute(domain)

#------------------------------------------------------
# Set domain parameters
#------------------------------------------------------

domain.set_name(project.scenario_name)
domain.set_datadir(project.output_run_time_dir)
domain.set_default_order(2) # Apply second order scheme
domain.set_minimum_storable_height(0.01) # Don't store anything less than 1cm
domain.set_store_vertices_uniquely(False)
domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])
domain.set_maximum_allowed_speed(0.1) # Allow a little runoff (0.1 is OK)

#-------------------------------------------------------------------------
# Setup boundary conditions 
#-------------------------------------------------------------------------
print 'Available boundary tags', domain.get_boundary_tags()
print 'domain id', id(domain)
print 'Reading Boundary file'
Bf = File_boundary(boundaries_dir_name + '.sww',
                  domain, time_thinning=1, use_cache=True, verbose=True)

print 'finished reading boundary file'

Br = Reflective_boundary(domain)
Bd = Dirichlet_boundary([tide,0,0])

print'set_boundary'
##domain.set_boundary({'back': Br,
##                     'side': Bf,
##                     'ocean': Bf})
domain.set_boundary({'back': Br,
                     'side': Bd,
                     'ocean': Bf}) 
print'finish set boundary'

#----------------------------------------------------------------------------
# Evolve system through time
#----------------------------------------------------------------------------

t0 = time.time()

for t in domain.evolve(yieldstep = 120, finaltime = 9000):
    domain.write_time()
    domain.write_boundary_statistics(tags = 'ocean')

#for t in domain.evolve(yieldstep = 120, finaltime = 9000):
#    domain.write_time()
#    domain.write_boundary_statistics(tags = 'ocean')
     
for t in domain.evolve(yieldstep = 60, finaltime = 28800
                       ,skip_initial_step = True):
    domain.write_time()
    domain.write_boundary_statistics(tags = 'ocean')    

for t in domain.evolve(yieldstep = 120, finaltime = 34800
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'ocean')    

x, y = domain.get_maximum_inundation_location()
q = domain.get_maximum_inundation_elevation()

print 'Maximum runup observed at (%.2f, %.2f) with elevation %.2f' %(x,y,q)

print 'That took %.2f seconds' %(time.time()-t0)

